
X_SHIFT = 1
Y_SHIFT = 4
MAX_MOVES = 9
NUM_LINES = 3
NUM_COLUMNS = 3
GAME_TITLE = '************ TIC TAC TOE ************'
GAME_CONTROLS = 'Use arrows to move, [SPACE] Draw, [Q] Quit'
BOARD_GRID = '──┼───┼──'
MSG_QUIT = 'Press any key to quit'



import copy


class GameLogic:
    """ Class of logic game
    """

    def __init__(self):
        self.x_pos = 1             # x position in matrix
        self.y_pos = 1             # y position in matrix
        self.count_moves = 0       # total of moves make along game
        self.game_states = []      # all game states of one game
        self.player_flag = False   # flag to know the player's turn
        self.end_game = False      # flag to find out if the game has come to an end

    def update_player(self) -> None:
        """ Updates which player to play """
        self.player_flag = not self.player_flag

    def has_victory(self, board, y, x) -> bool:
        """ Check if player has victory """
        return (
                (board[0][x] == board[1][x] == board[2][x])                    # check horizontal line
                or (board[y][0] == board[y][1] == board[y][2])                 # check vertical line
                or (x == y and board[0][0] == board[1][1] == board[2][2])      # check diagonal
                or (x + y == 2 and board[0][2] == board[1][1] == board[2][0])  # check diagonal
        )

    def update_move(self, board_game) -> None:
        """ Update the game logic for each move """
        # Update number of moves
        self.count_moves += 1
        # Save each game state
        self.game_states.append(copy.deepcopy(board_game))

    def update_input_positions(self, key) -> None:
        """ Update the game logic for each arrow input key """
        if key == "KEY_UP":
            self.y_pos = max(0, self.y_pos - 1)
        elif key == "KEY_DOWN":
            self.y_pos = min(2, self.y_pos + 1)
        elif key == "KEY_LEFT":
            self.x_pos = max(0, self.x_pos - 1)
        elif key == "KEY_RIGHT":
            self.x_pos = min(2, self.x_pos + 1)




import curses
import enum

from cursesUI import constants as const
from cursesUI.base import TextUI
# from .base import TextUI
from logic.game import GameLogic


class Cell(enum.Enum):
    EMPTY = " "
    CH_X = "X"
    CH_O = "O"

    def __str__(self):
        return f'{self.value}'


class GameUI(TextUI):
    """ Class GameUI
        @:param TextUI from base file (TextUI - courses)
    """

    def __init__(self):
        super().__init__()
        self.board_game = [[Cell.EMPTY.__str__()] * const.NUM_COLUMNS for _ in range(const.NUM_LINES)]
        self.game = GameLogic()

    def print_title(self) -> None:
        """ Print title of game """
        self.screen.addstr(1,
                           6,
                           const.GAME_TITLE,
                           curses.A_BOLD and curses.A_UNDERLINE)

    def print_board(self) -> None:
        """ Print board of game """
        self.screen.addstr(2,
                           3,
                           const.GAME_CONTROLS)
        self.screen.addstr(const.Y_SHIFT,
                           const.X_SHIFT + 4,
                           f'{self.board_game[0][0]} │ {self.board_game[0][1]} │ {self.board_game[0][2]}')
        self.screen.addstr(const.Y_SHIFT + 1,
                           const.X_SHIFT + 4,
                           const.BOARD_GRID)
        self.screen.addstr(const.Y_SHIFT + 2,
                           const.X_SHIFT + 4,
                           f'{self.board_game[1][0]} │ {self.board_game[1][1]} │ {self.board_game[1][2]}')
        self.screen.addstr(const.Y_SHIFT + 3,
                           const.X_SHIFT + 4,
                           const.BOARD_GRID)
        self.screen.addstr(const.Y_SHIFT + 4,
                           const.X_SHIFT + 4,
                           f'{self.board_game[2][0]} │ {self.board_game[2][1]} │ {self.board_game[2][2]}')

    def print_players(self) -> None:
        """ Print identification of players in each move """
        self.screen.addstr(const.Y_SHIFT + 6,
                           const.X_SHIFT + 4,
                           f'Player {Cell.CH_X.__str__()}',
                           curses.A_BOLD and curses.A_UNDERLINE if self.game.player_flag == 0 else 0)
        self.screen.addstr(const.Y_SHIFT + 7,
                           const.X_SHIFT + 4,
                           f'Player {Cell.CH_O.__str__()}',
                           curses.A_BOLD and curses.A_UNDERLINE if self.game.player_flag == 1 else 0)

    def print_result(self) -> None:
        """ Print the result of the game. """
        if self.game.has_victory(self.board_game, self.game.y_pos, self.game.x_pos):
            self.screen.addstr(const.Y_SHIFT + 2,
                               const.X_SHIFT + 20,
                               (
                                   f'Player {Cell.CH_X.__str__()} wins'
                                   if self.game.player_flag
                                   else f'Player {Cell.CH_O.__str__()} wins'
                               ),
                               curses.A_REVERSE)
            self.screen.addstr(const.Y_SHIFT + 3,
                               const.X_SHIFT + 17,
                               const.MSG_QUIT,
                               curses.A_BLINK)
            self.game.end_game = True
            curses.curs_set(0)
        elif self.game.count_moves == const.MAX_MOVES:
            self.screen.addstr(const.Y_SHIFT + 2,
                               const.X_SHIFT + 20,
                               'Nobody win',
                               curses.A_REVERSE)
            self.screen.addstr(const.Y_SHIFT + 3,
                               const.X_SHIFT + 17,
                               const.MSG_QUIT,
                               curses.A_BLINK)
            self.game.end_game = True
            curses.curs_set(0)

    def switch_player(self) -> None:
        """ Switch and print the current player """
        self.game.update_player()
        self.print_players()

    def draw(self) -> None:
        """ Draw all the TUI of game """
        curses.resize_term(20, 50)
        self.screen.border()
        self.print_title()
        self.print_board()
        x_step = 4
        y_step = 2
        if not self.game.end_game:
            self.print_players()
            self.screen.move(const.Y_SHIFT + self.game.y_pos * y_step,
                             const.X_SHIFT + self.game.x_pos * x_step + 4)
        else:
            self.print_result()

    def move(self) -> None:
        """ Make the update of each move in game. """
        # Get x and y of screen
        y, x = self.screen.getyx()
        # Validates if the move can be done (empty cell)
        if self.screen.inch(y, x) == ord(Cell.EMPTY.__str__()):
            # Draw the new move
            self.board_game[self.game.y_pos][self.game.x_pos] = (
                Cell.CH_O.__str__()
                if self.game.player_flag
                else Cell.CH_X.__str__()
            )
            # Update game logic of each move
            self.game.update_move(self.board_game)
            # Check for results
            self.print_result()
            # Switch player
            self.switch_player()

    def input(self, key) -> None:
        """ All the input options for play the game. """
        if not self.game.end_game:
            # Update input for arrows
            self.game.update_input_positions(key)
            if key == Cell.EMPTY.__str__():
                self.move()
            elif key == "q" or key == "Q":
                self.stop()
        else:
            if key:
                self.stop()




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['cursesUI', 'logic']

package_data = \
{'': ['*']}

install_requires = \
['certifi==2021.10.8', 'pip==21.2.4', 'setuptools==58.0.4', 'wheel==0.37.0']

setup_kwargs = {
    'name': 'tictactoe-marlene',
    'version': '0.1.4',
    'description': 'Tic Tac Toe game made with Python and Curses',
    'long_description': None,
    'author': 'Marlene Nunes',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.10,<4.0.0',
}


setup(**setup_kwargs)
